@@ -1,6 +1,6 @@
 /* strings -- print the strings of printable characters in files
    Copyright 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001,
-   2002 Free Software Foundation, Inc.
+   2002, 2003, 2004 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -39,10 +39,11 @@
    -o		Like -to.  (Some other implementations have -o like -to,
 		others like -td.  We chose one arbitrarily.)
 
-   --encoding={s,b,l,B,L}
-   -e {s,b,l,B,L}
-		Select character encoding: single-byte, bigendian 16-bit,
-		littleendian 16-bit, bigendian 32-bit, littleendian 32-bit
+   --encoding={s,S,b,l,B,L}
+   -e {s,S,b,l,B,L}
+		Select character encoding: 7-bit-character, 8-bit-character,
+		bigendian 16-bit, littleendian 16-bit, bigendian 32-bit,
+		littleendian 32-bit.
 
    --target=BFDNAME
 		Specify a non-default object file format.
@@ -80,11 +81,14 @@
 #endif
 #if O_BINARY
 #include <io.h>
-#define SET_BINARY(f) do { if (!isatty(f)) setmode(f,O_BINARY); } while (0)
+#define SET_BINARY(f) do { if (!isatty (f)) setmode (f,O_BINARY); } while (0)
 #endif
 #endif
 
-#define isgraphic(c) (ISPRINT (c) || (c) == '\t')
+#define STRING_ISGRAPHIC(c) \
+      (   (c) >= 0 \
+       && (c) <= 255 \
+       && ((c) == '\t' || ISPRINT (c) || (encoding == 'S' && (c) > 127)))
 
 #ifndef errno
 extern int errno;
@@ -95,10 +99,10 @@ extern int errno;
 
 #ifdef HAVE_FOPEN64
 typedef off64_t file_off;
-#define file_open(s,m) fopen64(s,m)
+#define file_open(s,m) fopen64(s, m)
 #else
 typedef off_t file_off;
-#define file_open(s,m) fopen(s,m)
+#define file_open(s,m) fopen(s, m)
 #endif
 
 /* Radix for printing addresses (must be 8, 10 or 16).  */
@@ -107,17 +111,17 @@ static int address_radix;
 /* Minimum length of sequence of graphic chars to trigger output.  */
 static int string_min;
 
-/* true means print address within file for each string.  */
-static boolean print_addresses;
+/* TRUE means print address within file for each string.  */
+static bfd_boolean print_addresses;
 
-/* true means print filename for each string.  */
-static boolean print_filenames;
+/* TRUE means print filename for each string.  */
+static bfd_boolean print_filenames;
 
-/* true means for object files scan only the data section.  */
-static boolean datasection_only;
+/* TRUE means for object files scan only the data section.  */
+static bfd_boolean datasection_only;
 
-/* true if we found an initialized data section in the current file.  */
-static boolean got_a_section;
+/* TRUE if we found an initialized data section in the current file.  */
+static bfd_boolean got_a_section;
 
 /* The BFD object file format.  */
 static char *target;
@@ -139,27 +143,22 @@ static struct option long_options[] =
   {NULL, 0, NULL, 0}
 };
 
-static void strings_a_section PARAMS ((bfd *, asection *, PTR));
-static boolean strings_object_file PARAMS ((const char *));
-static boolean strings_file PARAMS ((char *file));
-static int integer_arg PARAMS ((char *s));
-static void print_strings PARAMS ((const char *filename, FILE *stream,
-				  file_off address, int stop_point,
-				  int magiccount, char *magic));
-static void usage PARAMS ((FILE *stream, int status));
-static long get_char PARAMS ((FILE *stream, file_off *address,
-			      int *magiccount, char **magic));
+static void strings_a_section (bfd *, asection *, void *);
+static bfd_boolean strings_object_file (const char *);
+static bfd_boolean strings_file (char *file);
+static int integer_arg (char *s);
+static void print_strings (const char *, FILE *, file_off, int, int, char *);
+static void usage (FILE *, int);
+static long get_char (FILE *, file_off *, int *, char **);
 
-int main PARAMS ((int, char **));
+int main (int, char **);
 
 int
-main (argc, argv)
-     int argc;
-     char **argv;
+main (int argc, char **argv)
 {
   int optc;
   int exit_status = 0;
-  boolean files_given = false;
+  bfd_boolean files_given = FALSE;
 
 #if defined (HAVE_SETLOCALE)
   setlocale (LC_ALL, "");
@@ -170,9 +169,9 @@ main (argc, argv)
   program_name = argv[0];
   xmalloc_set_program_name (program_name);
   string_min = -1;
-  print_addresses = false;
-  print_filenames = false;
-  datasection_only = true;
+  print_addresses = FALSE;
+  print_filenames = FALSE;
+  datasection_only = TRUE;
   target = NULL;
   encoding = 's';
 
@@ -182,11 +181,11 @@ main (argc, argv)
       switch (optc)
 	{
 	case 'a':
-	  datasection_only = false;
+	  datasection_only = FALSE;
 	  break;
 
 	case 'f':
-	  print_filenames = true;
+	  print_filenames = TRUE;
 	  break;
 
 	case 'H':
@@ -196,18 +195,16 @@ main (argc, argv)
 	case 'n':
 	  string_min = integer_arg (optarg);
 	  if (string_min < 1)
-	    {
-	      fatal (_("invalid number %s"), optarg);
-	    }
+	    fatal (_("invalid number %s"), optarg);
 	  break;
 
 	case 'o':
-	  print_addresses = true;
+	  print_addresses = TRUE;
 	  address_radix = 8;
 	  break;
 
 	case 't':
-	  print_addresses = true;
+	  print_addresses = TRUE;
 	  if (optarg[1] != '\0')
 	    usage (stderr, 1);
 	  switch (optarg[0])
@@ -261,6 +258,7 @@ main (argc, argv)
 
   switch (encoding)
     {
+    case 'S':
     case 's':
       encoding_bytes = 1;
       break;
@@ -281,28 +279,28 @@ main (argc, argv)
 
   if (optind >= argc)
     {
-      datasection_only = false;
+      datasection_only = FALSE;
 #ifdef SET_BINARY
       SET_BINARY (fileno (stdin));
 #endif
       print_strings ("{standard input}", stdin, 0, 0, 0, (char *) NULL);
-      files_given = true;
+      files_given = TRUE;
     }
   else
     {
       for (; optind < argc; ++optind)
 	{
 	  if (strcmp (argv[optind], "-") == 0)
-	    datasection_only = false;
+	    datasection_only = FALSE;
 	  else
 	    {
-	      files_given = true;
-	      exit_status |= (strings_file (argv[optind]) == false);
+	      files_given = TRUE;
+	      exit_status |= strings_file (argv[optind]) == FALSE;
 	    }
 	}
     }
 
-  if (files_given == false)
+  if (!files_given)
     usage (stderr, 1);
 
   return (exit_status);
@@ -313,20 +311,18 @@ main (argc, argv)
    set `got_a_section' and print the strings in it.  */
 
 static void
-strings_a_section (abfd, sect, filearg)
-     bfd *abfd;
-     asection *sect;
-     PTR filearg;
+strings_a_section (bfd *abfd, asection *sect, void *filearg)
 {
   const char *file = (const char *) filearg;
 
   if ((sect->flags & DATA_FLAGS) == DATA_FLAGS)
     {
       bfd_size_type sz = bfd_get_section_size_before_reloc (sect);
-      PTR mem = xmalloc (sz);
+      void *mem = xmalloc (sz);
+
       if (bfd_get_section_contents (abfd, sect, mem, (file_ptr) 0, sz))
 	{
-	  got_a_section = true;
+	  got_a_section = TRUE;
 	  print_strings (file, (FILE *) NULL, sect->filepos, 0, sz, mem);
 	}
       free (mem);
@@ -336,48 +332,47 @@ strings_a_section (abfd, sect, filearg)
 /* Scan all of the sections in FILE, and print the strings
    in the initialized data section(s).
 
-   Return true if successful,
-   false if not (such as if FILE is not an object file).  */
+   Return TRUE if successful,
+   FALSE if not (such as if FILE is not an object file).  */
 
-static boolean
-strings_object_file (file)
-     const char *file;
+static bfd_boolean
+strings_object_file (const char *file)
 {
   bfd *abfd = bfd_openr (file, target);
 
   if (abfd == NULL)
-    {
-      /* Treat the file as a non-object file.  */
-      return false;
-    }
+    /* Treat the file as a non-object file.  */
+    return FALSE;
 
   /* This call is mainly for its side effect of reading in the sections.
      We follow the traditional behavior of `strings' in that we don't
      complain if we don't recognize a file to be an object file.  */
-  if (bfd_check_format (abfd, bfd_object) == false)
+  if (!bfd_check_format (abfd, bfd_object))
     {
       bfd_close (abfd);
-      return false;
+      return FALSE;
     }
 
-  got_a_section = false;
-  bfd_map_over_sections (abfd, strings_a_section, (PTR) file);
+  got_a_section = FALSE;
+  bfd_map_over_sections (abfd, strings_a_section, (void *) file);
 
   if (!bfd_close (abfd))
     {
       bfd_nonfatal (file);
-      return false;
+      return FALSE;
     }
 
   return got_a_section;
 }
 
-/* Print the strings in FILE.  Return true if ok, false if an error occurs.  */
+/* Print the strings in FILE.  Return TRUE if ok, FALSE if an error occurs.  */
 
-static boolean
-strings_file (file)
-     char *file;
+static bfd_boolean
+strings_file (char *file)
 {
+  if (get_file_size (file) < 1)
+    return FALSE;
+
   /* If we weren't told to scan the whole file,
      try to open it as an object file and only look at
      initialized data sections.  If that fails, fall back to the
@@ -391,7 +386,7 @@ strings_file (file)
 	{
 	  fprintf (stderr, "%s: ", program_name);
 	  perror (file);
-	  return false;
+	  return FALSE;
 	}
 
       print_strings (file, stream, (file_off) 0, 0, 0, (char *) 0);
@@ -400,11 +395,11 @@ strings_file (file)
 	{
 	  fprintf (stderr, "%s: ", program_name);
 	  perror (file);
-	  return false;
+	  return FALSE;
 	}
     }
 
-  return true;
+  return TRUE;
 }
 
 /* Read the next character, return EOF if none available.
@@ -418,11 +413,7 @@ strings_file (file)
    MAGICCOUNT is how many characters are in it.  */
 
 static long
-get_char (stream, address, magiccount, magic)
-     FILE *stream;
-     file_off *address;
-     int *magiccount;
-     char **magic;
+get_char (FILE *stream, file_off *address, int *magiccount, char **magic)
 {
   int c, i;
   long r = EOF;
@@ -454,6 +445,7 @@ get_char (stream, address, magiccount, magic)
 
   switch (encoding)
     {
+    case 'S':
     case 's':
       r = buf[0];
       break;
@@ -492,13 +484,8 @@ get_char (stream, address, magiccount, magic)
    Those characters come at address ADDRESS and the data in STREAM follow.  */
 
 static void
-print_strings (filename, stream, address, stop_point, magiccount, magic)
-     const char *filename;
-     FILE *stream;
-     file_off address;
-     int stop_point;
-     int magiccount;
-     char *magic;
+print_strings (const char *filename, FILE *stream, file_off address,
+	       int stop_point, int magiccount, char *magic)
 {
   char *buf = (char *) xmalloc (sizeof (char) * (string_min + 1));
 
@@ -518,14 +505,14 @@ print_strings (filename, stream, address, stop_point, magiccount, magic)
 	  c = get_char (stream, &address, &magiccount, &magic);
 	  if (c == EOF)
 	    return;
-	  if (c > 255 || c < 0 || !isgraphic (c))
+	  if (! STRING_ISGRAPHIC (c))
 	    /* Found a non-graphic.  Try again starting with next char.  */
 	    goto tryline;
 	  buf[i] = c;
 	}
 
       /* We found a run of `string_min' graphic characters.  Print up
-         to the next non-graphic character.  */
+	 to the next non-graphic character.  */
 
       if (print_filenames)
 	printf ("%s: ", filename);
@@ -570,7 +557,8 @@ print_strings (filename, stream, address, stop_point, magiccount, magic)
 #else
 # if !BFD_HOST_64BIT_LONG
 	    if (start != (unsigned long) start)
-	      printf ("%lx%8.8lx ", start >> 32, start & 0xffffffff);
+	      printf ("%lx%8.8lx ", (unsigned long) (start >> 32),
+		      (unsigned long) (start & 0xffffffff));
 	    else
 # endif
 #endif
@@ -586,7 +574,7 @@ print_strings (filename, stream, address, stop_point, magiccount, magic)
 	  c = get_char (stream, &address, &magiccount, &magic);
 	  if (c == EOF)
 	    break;
-	  if (c > 255 || c < 0 || !isgraphic (c))
+	  if (! STRING_ISGRAPHIC (c))
 	    break;
 	  putchar (c);
 	}
@@ -599,8 +587,7 @@ print_strings (filename, stream, address, stop_point, magiccount, magic)
    but allowing octal and hex numbers as in C.  */
 
 static int
-integer_arg (s)
-     char *s;
+integer_arg (char *s)
 {
   int value;
   int radix = 10;
@@ -636,16 +623,13 @@ integer_arg (s)
     p--;
 
   if (*p)
-    {
-      fatal (_("invalid integer argument %s"), s);
-    }
+    fatal (_("invalid integer argument %s"), s);
+
   return value;
 }
 
 static void
-usage (stream, status)
-     FILE *stream;
-     int status;
+usage (FILE *stream, int status)
 {
   fprintf (stream, _("Usage: %s [option(s)] [file(s)]\n"), program_name);
   fprintf (stream, _(" Display printable strings in [file(s)] (stdin by default)\n"));
@@ -657,8 +641,8 @@ usage (stream, status)
   -t --radix={o,x,d}        Print the location of the string in base 8, 10 or 16\n\
   -o                        An alias for --radix=o\n\
   -T --target=<BFDNAME>     Specify the binary file format\n\
-  -e --encoding={s,b,l,B,L} Select character size and endianness:\n\
-                            s = 8-bit, {b,l} = 16-bit, {B,L} = 32-bit\n\
+  -e --encoding={s,S,b,l,B,L} Select character size and endianness:\n\
+                            s = 7-bit, S = 8-bit, {b,l} = 16-bit, {B,L} = 32-bit\n\
   -h --help                 Display this information\n\
   -v --version              Print the program's version number\n"));
   list_supported_targets (program_name, stream);
