@@ -1,6 +1,6 @@
 /* gmon_io.h
-   
-   Copyright (C) 2000  Free Software Foundation, Inc.
+
+   Copyright 2000, 2001, 2002 Free Software Foundation, Inc.
 
 This file is part of GNU Binutils.
 
@@ -21,9 +21,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 #ifndef gmon_io_h
 #define gmon_io_h
 
-#include "bfd.h"
-#include "gmon.h"
-
 /* Some platforms need to put stdin into binary mode, to read
    binary files.  */
 #include "sysdep.h"
@@ -49,8 +46,14 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 extern int gmon_input;		/* What input did we see?  */
 extern int gmon_file_version;	/* File version are we dealing with.  */
 
-extern bfd_vma get_vma      PARAMS ((bfd *, bfd_byte *));
-extern void put_vma         PARAMS ((bfd *, bfd_vma, bfd_byte *));
+extern int gmon_io_read_vma PARAMS ((FILE *ifp, bfd_vma *valp));
+extern int gmon_io_read_32 PARAMS ((FILE *ifp, unsigned int *valp));
+extern int gmon_io_read PARAMS ((FILE *ifp, char *buf, size_t n));
+extern int gmon_io_write_vma PARAMS ((FILE *ifp, bfd_vma val));
+extern int gmon_io_write_32 PARAMS ((FILE *ifp, unsigned int val));
+extern int gmon_io_write_8 PARAMS ((FILE *ifp, unsigned int val));
+extern int gmon_io_write PARAMS ((FILE *ifp, char *buf, size_t n));
+
 extern void gmon_out_read   PARAMS ((const char *));
 extern void gmon_out_write  PARAMS ((const char *));
 
