@@ -69,6 +69,7 @@ enum option_values
   OPTION_CREF,
   OPTION_DEFSYM,
   OPTION_DEMANGLE,
+  OPTION_DEMANGLER,
   OPTION_DYNAMIC_LINKER,
   OPTION_EB,
   OPTION_EL,
@@ -315,6 +316,8 @@ static const struct ld_option ld_options[] =
       '\0', N_("SYMBOL=EXPRESSION"), N_("Define a symbol"), TWO_DASHES },
   { {"demangle", optional_argument, NULL, OPTION_DEMANGLE},
       '\0', N_("[=STYLE]"), N_("Demangle symbol names [using STYLE]"), TWO_DASHES },
+  { {"demangler", no_argument, NULL, OPTION_DEMANGLER},
+      '\0', "DSO:FUNCTION", N_("Set DSO and demangler function"), TWO_DASHES },
   { {"embedded-relocs", no_argument, NULL, OPTION_EMBEDDED_RELOCS},
       '\0', NULL, N_("Generate embedded relocs"), TWO_DASHES},
   { {"fini", required_argument, NULL, OPTION_FINI},
@@ -462,6 +465,7 @@ parse_args (unsigned argc, char **argv)
   struct option *really_longopts;
   int last_optind;
   enum report_method how_to_report_unresolved_symbols = RM_GENERATE_ERROR;
+  const char *demangler = NULL, *style = NULL;
 
   shortopts = xmalloc (OPTION_COUNT * 3 + 2);
   longopts = xmalloc (sizeof (*longopts) * (OPTION_COUNT + 1));
@@ -663,16 +667,10 @@ parse_args (unsigned argc, char **argv)
 	case OPTION_DEMANGLE:
 	  demangling = TRUE;
 	  if (optarg != NULL)
-	    {
-	      enum demangling_styles style;
-
-	      style = cplus_demangle_name_to_style (optarg);
-	      if (style == unknown_demangling)
-		einfo (_("%F%P: unknown demangling style `%s'"),
-		       optarg);
-
-	      cplus_demangle_set_style (style);
-	    }
+	    style = optarg;
+	  break;
+	case OPTION_DEMANGLER:
+	  demangler = optarg;
 	  break;
 	case 'I':		/* Used on Solaris.  */
 	case OPTION_DYNAMIC_LINKER:
@@ -737,6 +735,7 @@ parse_args (unsigned argc, char **argv)
 	  command_line.gc_sections = TRUE;
 	  break;
 	case OPTION_HELP:
+	  init_demangler (style, NULL, demangler);
 	  help ();
 	  xexit (0);
 	  break;
@@ -1240,6 +1239,8 @@ parse_args (unsigned argc, char **argv)
   if (link_info.unresolved_syms_in_shared_libs == RM_NOT_YET_SET)
     /* FIXME: Should we allow emulations a chance to set this ?  */
     link_info.unresolved_syms_in_shared_libs = how_to_report_unresolved_symbols;
+
+  init_demangler (style, NULL, demangler);
 }
 
 /* Add the (colon-separated) elements of DIRLIST_PTR to the
@@ -1383,5 +1384,8 @@ help (void)
   ldemul_list_emulation_options (stdout);
   printf ("\n");
 
+  printf (_("\nSupported demangler style: %s\n\n"),
+ 	  get_demangler_list ());
+
   printf (_("Report bugs to %s\n"), REPORT_BUGS_TO);
 }
